﻿using DE.TestFramework;
using Microsoft.Deployment.WindowsInstaller;
using System;
using System.Reflection;

namespace DE.TestFramework
{
    public enum ICELevel
    {
        Information = 3, Warning = 2, Error = 1
    }

    public class TestBase
    {

        private Session _session;
        public string _testName;

        public Session Session
        {
            get
            {
                return _session;
            }
        }
        public TestBase(Session session)
        {
            _session = session;
            _testName = this.GetType().Name.ToString();
        }

        public virtual void RunTests()
        {
            Publish(ICELevel.Information, "Executing ICE " + _testName);
            foreach (MethodInfo methodInformation in this.GetType().GetMethods())
            {
                foreach (Attribute attribute in methodInformation.GetCustomAttributes(true))
                {
                    Type test = attribute.GetType();
                    if (test.Name.Equals("ICETestAttribute"))
                    {

                        Publish(ICELevel.Information, "Invoking unit test: " + methodInformation.Name);
                        methodInformation.Invoke(this, new object[] { });
                    }
                }
            }
        }

        public void Publish(ICELevel iceLevel, string Description)
        {
            PublishMessage(iceLevel, Description);
        }

        public void Publish(ICELevel iceLevel, string Description, string HelpLocation)
        {
            PublishMessage(iceLevel, Description + "\t" + HelpLocation );
        }

        public void Publish(ICELevel iceLevel, string Description, string HelpLocation, string Table, string Column, string PrimaryKey)
        {
            PublishMessage(iceLevel, Description + "\t" + HelpLocation + "\t" + Table + "\t" + Column + "\t" + PrimaryKey);
        }

        public void Publish(ICELevel iceLevel, string Description, string HelpLocation, string Table, string Column, string[] PrimaryKeys)
        {
            string msgPrimaryKeys = string.Empty;

            foreach (string primaryKey in PrimaryKeys)
            {
                msgPrimaryKeys = msgPrimaryKeys + "\t" + primaryKey;
            }

            PublishMessage(iceLevel, Description + "\t" + HelpLocation + "\t" + Table + "\t" + Column + msgPrimaryKeys);
        }

        private void PublishMessage(ICELevel iceLevel, string FormattedMessage)
        {
            using (Record rec = new Record(1))
            {
                rec.SetString(0, _testName + "\t" + Convert.ToString((int)iceLevel) + "\t" + FormattedMessage);
                _session.Message(InstallMessage.User, rec);
            }
        }

    }

}
